/* Release 3.00 */
/*-------------------------------------------------------------------------*
*                                                                          *
*  stdio.h -  header file for stream input and output.                     *
*                                                                          *
*  COPYRIGHT (C) 1989..1991 Jensen and Partners. All Rights Reserved.      *
*                                                                          *
*--------------------------------------------------------------------------*/

#ifndef _STDIO_INC_
#define _STDIO_INC_

#ifndef  _SIZE_T
#define _SIZE_T
typedef unsigned size_t;
#endif

#ifndef NULL
#ifndef _fptr
#define NULL   0
#else
#define NULL   0L
#endif
#endif

#ifndef _STDARG_INC_
#include <stdarg.h>
#endif

#ifndef _ERRNOS_INC_
#define _ERRNOS_INC_

#ifdef __cplusplus
extern "C" {
#endif
#ifdef _mthread
#define errno *_errno__()
#define _doserrno *_errno__()

int *        _errno__(void);
#else
extern int   _doserrno;          /* global DOS error variable */
extern int   errno;              /* global error variable */
#endif
#ifdef __cplusplus
}
#endif
#endif

/* Number of files that can be open simultaneously */

#define OPEN_MAX 20			/* Total of 20 open files */
#define SYS_OPEN 20

/* define file control block */

#ifndef _IOB
typedef struct _iobuf {
    char *_ptr; 	 /* current char pointer */
    short _cnt; 	 /* buffer free space */
    short _size;	 /* buffer size */
    char *_base;	 /* buffer address */
    int   _flag;	 /* file attrib flag */
    int  _file; 	 /* file handle */
    int  _pback;	 /* ungot char if no buffer */
    long _sem;
    char _ctrl;
    char _scnt;
} FILE;

#ifdef __cplusplus
extern "C" {
#endif
extern FILE  _iob[OPEN_MAX];
#ifdef __cplusplus
}
#endif
#define IOSEM(st)  &(st)->_sem
#define _IOB
#endif

/* Buffer type to be used as 3rd argument for "setvbuf" function */

#define _IOFBF	0
#define _IOLBF	1
#define _IONBF	2

/* End-of-file constant definition  */

#define EOF	(-1)


/* Standard I/O predefined streams */

#ifndef _WINDOWS
#define stdin	(&_iob[0])
#define stdout	(&_iob[1])
#define stderr	(&_iob[2])
#define stdaux	(&_iob[3])
#define stdprn	(&_iob[4])
#endif

/* flags bit definitions */

#define _F_RDWR 0x0003			/* Read/write  */
#define _F_READ 0x0001			/* Read only file */
#define _F_WRIT 0x0002			/* Write only file */
#define _F_UBUF 0x0004			/* User allocated buffer */
#define _F_APP	0x0008			/* Append data */
#define _F_ERR	0x0010			/* Error */
#define _F_EOF	0x0020			/* EOF */
#define _F_BIN	0x0040			/* Binary file */
#define _F_IN	0x0080			/* Data is incoming */
#define _F_OUT	0x0100			/* Data is outgoing */
#define _F_DEV	0x0200			/* File is terminal */
#define _F_RST	0x0400			/* Newly opened stream */
#define _F_LBUF 0x0800			/* line buffered stream */
#define _F_CTZ	0x1000

/* Default buffer size use by "setbuf" function */

#define BUFSIZ	512			/* Buffer size for stdio */

/* Constants to be used as 3rd argument for "fseek" function */

#define SEEK_SET	0
#define SEEK_CUR	1
#define SEEK_END	2

#ifndef _FPOS_T_DEF
typedef long fpos_t;
#define _FPOS_T_DEF
#endif

#ifdef __cplusplus
extern "C" {
#endif
void	  clearerr(FILE *_st);
int	  fclose(FILE *_st);
int	  fflush(FILE *_st);
int	  fgetc(FILE *_st);
char *    fgets(char *s, int _n, FILE *_st);
FILE *    fopen(const char *_path, const char *_mode);
int	  fprintf(FILE *_st, const char *_format, ...);
int	  fputc(int _c, FILE *_st);
int	  fputs(const char *_s, FILE *_st);
size_t	  fread(void *_ptr, size_t _size, size_t _n, FILE *_st);
FILE *    freopen(const char *_path, const char *_mode,
		  FILE *_oldfile);
int	  fscanf(FILE *_st, const char *_format, ...);
int	  fseek(FILE *_st, long _offset, int _whence);
long	  ftell(FILE *_st);
size_t	  fwrite(const void *_ptr, size_t _size, size_t _n,
		 FILE *_st);
char *    gets(char *_s);
void	  perror(const char *_s);
#ifndef _WINDOWS
int	  printf(const char *_format, ...);
#endif
int	  puts(const char *_s);
int	  rename(const char *_oldname, const char *_newname);
void	  rewind(FILE *_st);
int       rmtmp(void);
#ifndef _WINDOWS
int	  scanf(const char *_format, ...);
#endif
void	  setbuf(FILE *_st, char *_buf);
int	  setvbuf(FILE *_st, char *_buf, int _type, size_t _size);
int	  sprintf(char *_s, const char *_format, ...);
int	  sscanf(const char *_s, const char *_format, ...);
char *    strerror(int _errnum);
FILE *    tmpfile(void);
char *    tmpnam(char *_s);
char *    tempnam(char *_dir, char *_prefix);
int	  ungetc(int _c, FILE *_st);
int	  vfprintf(FILE *_st, const char *_format, va_list _argptr);
int	  vfscanf(FILE *_st, const char *_format, va_list _argptr);
#ifndef _WINDOWS
int	  vprintf(const char *_format, va_list _argptr);
int	  vscanf(const char *_format, va_list _argptr);
#endif
int	  vsprintf(char *_s, const char *_format, va_list _argptr);
int	  vsscanf(const char *_s, const char *_format, va_list _argptr);

int       fcloseall(void);
FILE *    fdopen(int _handle, char *_type);
int       fgetchar(void);
int       fgetpos(FILE *_st, fpos_t *_fp);
int       fsetpos(FILE *_st, const fpos_t *_fp);
int       flushall(void);
int       fputchar(int _c);
int       getw(FILE *_st);
int       putw(int _w, FILE *_st);
char *    _strerror(const char *_s);
int       unlink(const char *_path);

/*   common functions normally implementetd as macros */

#ifndef _WINDOWS
int  putchar(int _c);
int  getchar(void);
#endif
int  putc(int _c, FILE *_st);
int  getc(FILE *_st);
int  remove(const char *_path);
int  fileno(FILE *_st);
int  feof(FILE *_st);
int  ferror(FILE *_st);
#ifdef __cplusplus
}
#endif

#ifndef __STDC__

#ifdef __cplusplus
extern "C" {
#endif
size_t	 hfread(void huge *_ptr, size_t _size, size_t _n, FILE *_st);
size_t	 hfwrite(const void huge *_ptr, size_t _size, size_t _n,
		 FILE *_st);
#ifdef __cplusplus
}
#endif
		
#ifdef _HUGE_
#define fread(_ptr, _size, _n, _st)    hfread(_ptr, _size, _n, _st)
#define fwrite(_ptr, _size, _n, _st)   hfwrite(_ptr, _size, _n, _st)
#endif


#ifndef _IO_MTF
#define ferror(f)	((f)->_flag & _F_ERR)
#define feof(f) 	((f)->_flag & _F_EOF)
#define fileno(f)	((f)->_file)
#define getc(f)         fgetc(f)
#define putc(c,f)       fputc(c,f)
#ifndef __cplusplus
#define remove(path)    unlink(path)
#else
inline int remove(const char *_path) { return unlink(_path); }
#endif
#ifndef _WINDOWS
#define getchar()       fgetc(stdin)
#define putchar(c)      fputc((c), stdout)
#endif
#endif
#endif

#define P_tmpdir        "\\"
#define L_tmpnam        sizeof(P_tmpdir)+13
#define FOPEN_MAX	15
#define FILENAME_MAX	80
#define TMP_MAX      10000
#endif
