/* Release 3.00 */
/*-------------------------------------------------------------------------*
*                                                                          *
*  stdlib.h - definitions for common variables, types and functions.       *
*                                                                          *
*  COPYRIGHT (C) 1989..1991 Jensen and Partners. All Rights Reserved.      *
*                                                                          *
*                                                                          *
*--------------------------------------------------------------------------*/

#ifndef _STDLIB_INC_
#define _STDLIB_INC_

#ifndef _SIZE_T
#define _SIZE_T
typedef unsigned size_t;
#endif

#ifndef _WCHAR_T
#define _WCHAR_T
typedef unsigned char wchar_t;
#endif

#ifndef NULL
#ifndef _fptr
#define NULL   0
#else
#define NULL   0L
#endif
#endif

#define MB_CUR_MAX 1

#ifndef _LDIV_T
#define _LDIV_T
typedef struct {
   long    quot;
   long    rem;
} ldiv_t;
#endif

#ifndef _DIV_T
#define _DIV_T
typedef struct {
   int    quot;
   int    rem;
} div_t;
#endif

#ifndef _ERRNOL_INC_
#define _ERRNOL_INC_

#ifdef __cplusplus
extern "C" {
#endif
#ifdef _mthread
#define errno *_errno__()
#define _doserrno *_errno__()

int *_errno__(void);
#else
extern int   _doserrno;          /* global DOS error variable */
extern int   errno;              /* global error variable */
#endif
#ifdef __cplusplus
}
#endif
#endif

#ifdef __cplusplus
extern "C" {
#endif
void	      abort(void);
int           abs(int _num);
int           atexit(void (*_func)(void));
double        atof(const char *_nptr);
long double   atofl(const char *_nptr);
int           atoi(const char *_nptr);
long          atol(const char *_nptr);
unsigned long atoul(const char *_nptr);
void *        bsearch(const void *_key, const void *_base,
		      size_t _nmemb, size_t _size,
		      int ( *_compare)(const void *_e1, const void *_e2));
void *        calloc(size_t _num, size_t _size);
void          delay(int _tm);
div_t         div(int _numer, int _denom);
char *        ecvt(double _value, int _digits, int *_dec, int *_sign);
void	      _exit(int _status);
void	      exit(int _code);
char *        fcvt(double _value, int _digits, int *_dec, int *_sign);
void          free(void *_block);
char *        gcvt(double _value, int _ndec, char *_buf);
char *        gcvtl(long double _value, int _ndec, char *_buf);
char *        getenv(const char *_name);
char *        itoa(int _value, char *_string, int _radix);
long	      labs(long _j);
ldiv_t        ldiv(long _numer, long _denom);
#ifndef _HUGE_
char *        lfind(const char *_key, const char *_base,
		    unsigned *_num, unsigned _width,
		    int ( *_compare)(const void *_e1, const void *_e2));
#endif
unsigned long _lrotl(unsigned long _val, int _count);
unsigned long _lrotr(unsigned long _val, int _count);
#ifndef _HUGE_
char *        lsearch(const char *_key, char *_base,
		      unsigned *_num, unsigned _width,
		      int ( *_compare)(const void *_e1, const void *_e2));
#endif		
char *        ltoa(long _value, char *_s, int _radix);
void          _makepath(char *_path, const char *_drive, const char *_dir,
	                const char *_name, const char *_ext);
void *        malloc(size_t _size);
int           mblen(const char *_s, size_t _n);
size_t        mbstowcs(wchar_t *_pwcs, const char *_s, size_t _n);
int           mbtowc(wchar_t *_pwc, const char *_s, size_t _n);
void       (* onexit(void ( *_func)(void)))();
int           putenv(const char *_name);
#ifndef _HUGE_
void	      qsort(void *_base, size_t _nmemb, size_t _size,
		    int (*_compare)(const void *_e1, const void *_e2));
#endif		
int	      rand(void);
void          randomize(void);
void *        realloc(void *_block, size_t _size);
unsigned      _rotl(unsigned _value, int _count);
unsigned      _rotr(unsigned _value, int _count);

void          _searchenv(const char *_name, const char *_env, char *_path);
void          _splitpath(const char *_path, char *_drive, char *_dir,
                        char *_name, char *_ext);
void	      srand(unsigned _seed);
double	      strtod(const char *_s, char **_endptr);
long	      strtol(const char *_s, char **_endptr, int _base);
unsigned long strtoul(const char *_s, char **_endptr, int _base);
void          swab(const char *_from, char *_to, int _num);
int	      system(const char *_string);
char *        ultoa(unsigned long _value, char *_s, int _radix);
int           wctomb(char *_s, wchar_t _wchar);
size_t        wcstombs(char *_s, const wchar_t *_pwcs, size_t _n);


/* Variables */

extern	unsigned int	 _fmode;
extern	char	       * sys_errlist[38];
extern	int		 sys_nerr;
#ifdef __cplusplus
}
#endif

#define EXIT_SUCCESS	0
#define EXIT_FAILURE	0xFF

#ifndef max
#define max(a,b)	(((a) > (b)) ? (a) : (b))
#endif
#ifndef min
#define min(a,b)	(((a) < (b)) ? (a) : (b))
#endif

#define random(num)	(rand() % (num))
#define RAND_MAX 32767


#define _MAX_PATH  144      /* maximum length of full pathname */
#define _MAX_DRIVE   4      /* maximum length of drive component */
#define _MAX_DIR   130      /* maximum length of path component */
#define _MAX_FNAME   9      /* maximum length of file name component */
#define _MAX_EXT     5      /* maximum length of extension component */


#endif
